package EDU.bmrb.net;

import java.lang.*;
import java.util.*;
import java.net.*;

/** This class provides the ability to behave like a web browser submitting
  * a HTML form to a Common Gateway Interface (CGI) program on a web
  * server.  When a user on a web browser fills in the blanks of a web
  * form and hits 'submit', the information filled in to the form is
  * sent to the server in the body of the HTTP message.  The standard
  * JDK class HttpURLConnection provides all the required tools for dealing
  * with the fields in the header of the HTTP message, but it does not 
  * have any nice abstractions of the body of the message.  This new
  * <CODE>HttpFormURLConnection</CODE> class provides the extra code to 
  * handle the body of a form.
  * <P>
  * Let's take an example.  Let's say we had a HTML form that was defined like this:
  * <PRE>
  *     &lt;FORM ENCTYPE="multipart/form-data" METHOD="POST"
  *             ACTION="http://some.host.name/cgi-bin/runme"&gt;
  *         Fill out the following form before crossing the bridge of fate:
  *         &lt;P&gt;
  *         What is your name:
  *             &lt;INPUT TYPE="text" NAME="the_name" SIZE=40&gt; &lt;BR&gt;
  *         What is your quest in life: (Please submit attached file)
  *             &lt;INPUT TYPE="file" NAME="the_quest" SIZE=30&gt; &lt;BR&gt;
  *         What is your favorite color:
  *             &lt;SELECT NAME="the_color"&gt;
  *                  &lt;OPTION&gt;red&lt;/OPTION&gt;
  *                  &lt;OPTION&gt;green&lt;/OPTION&gt;
  *                  &lt;OPTION&gt;yellow&lt;/OPTION&gt;
  *                  &lt;OPTION&gt;No wait, it's blue!&lt;/OPTION&gt;
  *             &lt;/SELECT&gt;
  *         &lt;INPUT TYPE=SUBMIT&gt;
  *     &lt;/FORM&gt;
  * </PRE>
  * <P>
  * Now, let's say you wanted to have your Java program act as if it were
  * the browser, and have it submit this information to the web server.
  * For this example, let's say you wanted to fill it out like this:
  * <PRE>
  *     <FORM ENCTYPE="multipart/form-data" METHOD="POST"
  *             ACTION="bogus-dont-try-to-run-this">
  *         Fill out the following form before crossing the bridge of fate:
  *         <P>
  *         What is your name:
  *             <INPUT TYPE="text" NAME="the_name" SIZE=40 VALUE="Sir Robin"> <BR>
  *         What is your quest in life: (Please submit attached file)
  *             <INPUT TYPE="file" NAME="the_quest" SIZE=30 VALUE="grail_seek.txt"> <BR>
  *         What is your favorite color:
  *             <SELECT NAME="the_color">
  *                  <OPTION>red</OPTION>
  *                  <OPTION>green</OPTION>
  *                  <OPTION>yellow</OPTION>
  *                  <OPTION SELECTED>No wait, it's blue!</OPTION>
  *             </SELECT>
  *         <INPUT TYPE=SUBMIT>
  *     </FORM>
  * </PRE>
  * <P>
  * Here's how you could do it, using this class: 
  * <PRE>
  *     // PLEASE NOTE: exception catching has been left out of this on purpose
  *     // to keep the example brief.  In reality there are many network errors
  *     // that could be thrown here that I am not showing.
  *     HttpFormURLConnection the_connection = new HttpFormURLConnection( TODO );
  *
  *     // Fill out the_name field:
  *     the_connection.setNameValuePair( "the_name", "Sir Robin" );
  *     // Fill out the_color field:
  *     the_connection.setNameValuePair( "the_color", "No wait, it's blue!" );
  *     // Fill out the_quest field:
  *     // Let's assume you have already read the contents of the file called "grail_seek.txt"
  *     // into a Java String variable called "file_contents" (this step not shown).
  *     the_connection.setNameValuePair( "the_quest", "grail_seek.txt", file_contents);
  *     // Now you can send it:
  *     the_connection.submit();
  *
  *     // Presumably you did this so that you could read the result sent back by the
  *     // the web server.  Now at this point you can fall back on the standard
  *     // HttpURLConnection methods to do this, since the returned result is much
  *     // simpler to deal with in a 'raw' fashion than the form was:
  *     TODO - write this part.
  * </PRE>
  * <P>
  * (As of this writing, only the POST method is supported by this class,
  * since that is the easiest way to encompass all functionality.  The
  * HTTP GET method can only provide a subset of the POST functionality, so
  * there is no incentive to bother supporting it in this class (although
  * this might be done later if need be.).)
  * <P>
  * <CODE>HttpFormURLConnection</CODE> is derived from the standard JDK
  * class <CODE>HttpURLConnection</CODE>.
  */
